"use strict";

var AbstractAdminPanel = require('./AbstractAdminPanel');
var PrivilegeType = require('../../map/data/PrivilegeType');
var ConfigurationType = require('../../ConfigurationType');

var Functions = require('../../Functions');
var GuiConnector = require('../../GuiConnector');

var logger = require('../../logger');

var Promise = require("bluebird");
var xss = require("xss");

/**
 *
 * @param {Configuration} [params.configuration]
 * @param {HTMLElement} params.element
 * @param {Project} params.project
 * @param {string} params.panelName
 * @param params.parent
 *
 * @constructor
 * @extends AbstractAdminPanel
 */
function ConfigurationAdminPanel(params) {
  AbstractAdminPanel.call(this, params);

  var self = this;
  $(self.getElement()).removeClass("pre-scrollable");
  $(self.getElement()).addClass("minerva-configuration-tab");

  self._createGui();
}

ConfigurationAdminPanel.prototype = Object.create(AbstractAdminPanel.prototype);
ConfigurationAdminPanel.prototype.constructor = ConfigurationAdminPanel;

/**
 *
 * @private
 */
ConfigurationAdminPanel.prototype._createGui = function () {
  var self = this;
  self.getGuiUtils().initTabContent(self);


  $(self.getElement()).on("click", "[name='saveOption']", function () {
    var button = this;
    return self.saveOption($(button).attr("data")).then(null, GuiConnector.alert);
  });
  $(self.getElement()).on("input", ".minerva-color-input", function () {
    var input = this;
    var value = $(input).val();
    var type = $(input).attr("data");
    if (value.length !== 6) {
      value = "FFFFFF";
    }
    $("[name='edit-color-" + type + "']", self.getElement()).css("background-color", "#" + value);
  });
  $(self.getElement()).on("click", ".minerva-color-button", function () {
    var button = this;
    var value = $(button).css("background-color");
    var type = $(button).attr("data");
    // noinspection JSUnusedGlobalSymbols
    var colorPicker = $(button).parent().spectrum({
      color: value,
      move: function (color) {
        var value = color.toHexString().replace("#", '');
        $("[name='edit-" + type + "']", self.getElement()).val(value);
        $("[name='edit-color-" + type + "']", self.getElement()).css("background-color", "#" + value);
      },
      hide: function () {
        colorPicker.spectrum("destroy");
      }
    });
    return new Promise.delay(1).then(function () {
      colorPicker.show();
      colorPicker.spectrum("show");
    });
  });
};

/**
 *
 * @param {ConfigurationOption[]} options
 * @param {string} type
 */
ConfigurationAdminPanel.prototype.createOptionsTable = function (options, type) {
  var self = this;

  var configurationDiv = Functions.createElement({
    type: "div",
    className: "pre-scrollable"
  });
  var configurationTable = Functions.createElement({
    type: "table",
    name: "configurationTable",
    className: "display",
    style: "width:100%"
  });
  configurationDiv.appendChild(configurationTable);

  // noinspection JSUnusedGlobalSymbols
  var dataTable = $(configurationTable).DataTable({
    columns: [{
      title: 'Name'
    }, {
      title: 'Value'
    }, {
      title: 'Save',
      orderable: false
    }],
    order: [[1, "asc"]]
  });

  var data = [];

  for (var i = 0; i < options.length; i++) {
    var option = options[i];
    if (option.getGroup() === type) {
      var rowData = self.optionToTableRow(option);
      data.push(rowData);
    }
  }

  dataTable.clear().rows.add(data).draw();

  self.getGuiUtils().addTab(self, {name: type, content: configurationDiv});

  return self.bindUserGuiPreference({
    jQueryObject: $(configurationTable),
    event: 'length.dt',
    preferenceName: 'admin-configuration-datatable-length',
    defaultValue: '10',
    getter: function () {
      return $(configurationTable).DataTable().page.len() + '';
    },
    setter: function (value) {
      return $(configurationTable).DataTable().page.len(value).draw();
    }
  });
};

/**
 *
 * @returns {Promise}
 */
ConfigurationAdminPanel.prototype.init = function () {
  var self = this;
  return AbstractAdminPanel.prototype.init.call(this).then(function () {
    return self.getServerConnector().getLoggedUser();
  }).then(function (user) {
    var configuration = self.getConfiguration();
    var privilege = configuration.getPrivilegeType(PrivilegeType.CONFIGURATION_MANAGE);
    if (user.hasPrivilege(privilege)) {
      return self.setOptions(configuration.getOptions());
    } else {
      self.disablePanel("You have no privilege to manage configuration");
    }
  });
};

/**
 *
 * @param {ConfigurationOption[]} options
 *
 * @returns {Promise}
 */
ConfigurationAdminPanel.prototype.setOptions = function (options) {
  var self = this;

  var categories = {"": true};
  var promises = [];
  for (var i = 0; i < options.length; i++) {
    var option = options[i];

    var group = option.getGroup();
    if (categories[group] === undefined && group !== undefined) {
      categories[group] = true;
      promises.push(self.createOptionsTable(options, group));
    }
  }
  return Promise.all(promises);
};

/**
 *
 * @param {ConfigurationOption} option
 * @returns {Array}
 */
ConfigurationAdminPanel.prototype.optionToTableRow = function (option) {
  var value = option.getValue();
  var row = [];
  var editOption;

  if (option.getValueType() === "STRING" ||
    option.getValueType() === "INTEGER" ||
    option.getValueType() === "DOUBLE" ||
    option.getValueType() === "EMAIL" ||
    option.getValueType() === "PASSWORD" ||
    option.getValueType() === "URL") {
    editOption = "<input name='edit-" + option.getType() + "' value='" + value + "'/>";
  } else if (option.getValueType() === "TEXT") {
    editOption = "<textarea name='edit-" + option.getType() + "'>" + xss(value) + "</textarea>";
  } else if (option.getValueType() === "BOOLEAN") {
    var checked = "";
    if (value.toLowerCase() === "true") {
      checked = " checked ";
    }
    editOption = "<input type='checkbox' name='edit-" + option.getType() + "' " + checked + " />";
  } else if (option.getValueType() === "COLOR") {
    editOption = "<div>" +
      "<input class='minerva-color-input' name='edit-" + option.getType() + "' data='" + option.getType() + "' value='" + value + "'/>" +
      "<button class='minerva-color-button' name='edit-color-" + option.getType() + "' data='" + option.getType() + "' style='background-color: #" + value + "'>&nbsp</button>" +
      "</div>";
  } else {
    logger.warn("Don't know how to handle: " + option.getValueType());
    editOption = "<input name='edit-" + option.getType() + "' value='" + value + "' readonly/>";
  }
  row[0] = option.getCommonName();
  row[1] = editOption;
  row[2] = "<button name='saveOption' data='" + option.getType() + "' ><i class='fa fa-save' style='font-size:17px'></i></button>";
  return row;
};

/**
 *
 * @param {string} type
 * @returns {Promise}
 */
ConfigurationAdminPanel.prototype.saveOption = function (type) {
  var self = this;
  return self.getServerConnector().getConfiguration().then(function (configuration) {
    var option = configuration.getOption(type);
    var element = $("[name='edit-" + type + "']", self.getElement());
    var value;
    if (element.is(':checkbox')) {
      if (element.is(':checked')) {
        value = "true";
      } else {
        value = "false";
      }
    } else {
      value = element.val();
    }
    option.setValue(value);
    return self.getServerConnector().updateConfigurationOption(option);
  }).then(function () {
    if (type === ConfigurationType.TERMS_OF_USE) {
      return new Promise(function (resolve) {
        var html = '<form><span>Do you want to reset user the status of ToS acceptance for all users of the platform?</span><br></form>';
        $(html).dialog({
          modal: true,
          title: 'Status of users ToS acceptance',
          close: function () {
            $(this).dialog('destroy').remove();
            resolve(false);
          },
          buttons: {
            'YES': function () {
              $(this).dialog('destroy').remove();
              resolve(true);
            },
            'NO': function () {
              $(this).dialog('destroy').remove();
              resolve(false);
            }
          }
        });
      }).then(function (status) {
        if (status) {
          return self.getServerConnector().resetUserTos();
        } else {
          console.log("no");
        }
      });
    }
  });
};

/**
 *
 */
ConfigurationAdminPanel.prototype.destroy = function () {
  var self = this;
  var tables = $("[name='configurationTable']", self.getElement());
  for (var i = 0; i < tables.length; i++) {
    if ($.fn.DataTable.isDataTable(tables[i])) {
      $(tables[i]).DataTable().destroy();
    }
  }

};

module.exports = ConfigurationAdminPanel;
